package org.bouncycastle.asn1.pkcs;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.asn1.util.*;
/**
 * a PKCS#7 signer info object.
 */
public class SignerInfo
    implements DEREncodable, PKCSObjectIdentifiers
{
    private DERInteger              version;
    private X509Name                issuer;
    private DERInteger              serialNumber;
	private DERConstructedSequence  issuerAndSerialNumber;
    private AlgorithmIdentifier     digestAlgorithm;
    private DERConstructedSet       authenticatedAttributes;
    private AlgorithmIdentifier     digestEncryptionAlgorithm;
    private DEROctetString          encryptedDigest;
    private DERConstructedSet       unauthenticatedAttributes;


    public SignerInfo( DERConstructedSequence  seq )
    {

		int i = 0;

                version = (DERInteger)seq.getObjectAt(i++);

		// extract issuer & serialNumber
		DERConstructedSequence issuerAndSerialNumber = (DERConstructedSequence)seq.getObjectAt(i++);

                //changed
                issuer = new X509Name( (DERConstructedSequence)issuerAndSerialNumber.getObjectAt(0) );
		
                serialNumber = (DERInteger)issuerAndSerialNumber.getObjectAt(1);
                
                //changed
		digestAlgorithm = new AlgorithmIdentifier( (DERConstructedSequence)seq.getObjectAt(i++) );

		// authenticatedAttributes is optional !
                //changed
		//if (seq.getObjectAt(0) instanceof DERTaggedObject) {

                try {
                    
                    DERTaggedObject tagobj = (DERTaggedObject)seq.getObjectAt(i);

                    if (!tagobj.isEmpty()) {

                        DERConstructedSequence s = (DERConstructedSequence)tagobj.getObject();
                        authenticatedAttributes = new DERConstructedSet();

                        for (int a = 0; a != s.getSize(); a++)
                        {
                            authenticatedAttributes.addObject((DEREncodable)s.getObjectAt(a));
                        }
                    }

			i++;
                        
                        System.out.println("tagged attribute successfully read");
		} catch (Exception e) {
                    //no tagged attributes (cast exception)
                }

                //changed
                //if ( seq.getObjectAt(i) instanceof AlgorithmIdentifier ) {
		//	digestEncryptionAlgorithm = (AlgorithmIdentifier)seq.getObjectAt(i++);
		//} else {
		
                digestEncryptionAlgorithm = new AlgorithmIdentifier( (DERConstructedSequence)seq.getObjectAt(i++) );
		//}

		encryptedDigest = (DEROctetString)seq.getObjectAt(i++);

		// unauthenticatedAttributes is optional !
		if (seq.getObjectAt(0) instanceof DERTaggedObject) {

                    DERTaggedObject tagobj = (DERTaggedObject)seq.getObjectAt(i);

                    if (!tagobj.isEmpty()) {

                        DERConstructedSequence s = (DERConstructedSequence)tagobj.getObject();
                        unauthenticatedAttributes = new DERConstructedSet();

                        for (int a = 0; a != s.getSize(); a++)
                        {
                            unauthenticatedAttributes.addObject((DEREncodable)s.getObjectAt(a));
                        }
                    }

			i++;
		}



    }

	public SignerInfo( X509Name issuer, DERInteger serial, AlgorithmIdentifier algorithm,
					   DERConstructedSet authAttrs, AlgorithmIdentifier digestEncryptionAlgorithm, 
					   DEROctetString encryptedDigest, DERConstructedSet unauthAttrs )
	{
		this.version = new DERInteger( 1 );

		this.issuerAndSerialNumber = new DERConstructedSequence();
		this.issuerAndSerialNumber.addObject( issuer );
		this.issuerAndSerialNumber.addObject( serial );

		this.digestAlgorithm = algorithm;

		if ( authAttrs != null )
				this.authenticatedAttributes = authAttrs;

		this.digestEncryptionAlgorithm = digestEncryptionAlgorithm;

		this.encryptedDigest = encryptedDigest;

		if ( unauthAttrs != null )
				this.unauthenticatedAttributes = authAttrs;

	}

    public DERInteger getVersion()
    {
        return version;
    }

    public void setIssuer( X509Name _issuer ) {
		issuer = _issuer;
	}

	public void setSerial( int serial ) {
		serialNumber = new DERInteger( serial );
	}

    public DERConstructedSet getAuthenticatedAttributes() {
        return authenticatedAttributes;
    }

    public void setAuthenticatedAttributes( DERConstructedSet attr ) {
		authenticatedAttributes = attr;
	}

    public DEROctetString getEncryptedDigest() {
        return encryptedDigest;
    }

    public void setEncryptedDigest( DEROctetString digest) {
        encryptedDigest = digest;
    }

    /**
     * <pre>
     * see PKCS#9 for details
     *
     * SignerInfo ::= SEQUENCE { version Version, issuerAndSerialNumber IssuerAndSerialNumber, digestAlgorithm DigestAlgorithmIdentifier, authenticatedAttributes [0] IMPLICIT Attributes OPTIONAL, digestEncryptionAlgorithm DigestEncryptionAlgorithmIdentifier, encryptedDigest EncryptedDigest, unauthenticatedAttributes [1] IMPLICIT Attributes OPTIONAL }
     *
     * </pre>
     * @return  */
    public DERObject getDERObject()
    {
        DERConstructedSequence  seq = new DERConstructedSequence();

        seq.addObject(version);
        
        seq.addObject(issuerAndSerialNumber);
        
        seq.addObject(digestAlgorithm);
		
        if (authenticatedAttributes != null)
        {
        	seq.addObject( new DERTaggedObject( false, 0, authenticatedAttributes));
        }

        seq.addObject(digestEncryptionAlgorithm);
        seq.addObject(encryptedDigest);

        if (unauthenticatedAttributes != null)
        {
        	seq.addObject( new DERTaggedObject( false, 1, unauthenticatedAttributes));
        }

        return seq;
    }
}
